const CLASS_ID = Components.ID("FEFAD2B4-DBC9-11DC-B22F-59AC56D89593");
const CLASS_NAME = "Mahalo Share Timer";
const CONTRACT_ID = "@mahalo.com/mahalo-share-timer;1";

function MahaloShareTimer() {
	var obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
	obs.addObserver(this, "xpcom-shutdown", false);
	
	this._checkTimer = Components.classes["@mozilla.org/timer;1"];
	this._checkTimer = this._checkTimer.createInstance(Components.interfaces.nsITimer);

	this._windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
    this._checkTimer.initWithCallback(this, 1000 * 5, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
    
    return true;
}

// This is the implementation of your component.
MahaloShareTimer.prototype = {
	_timer: null,
	_checkTimer : null,
	_windowManager : null,
	_window : null,
	checkTime : 60 * 1000, // Once per minute.
	
	notify : function(aTimer) {
		var file = Components.classes["@mozilla.org/file/directory_service;1"]
		                     .getService(Components.interfaces.nsIProperties)
		                     .get("ProfD", Components.interfaces.nsIFile);
		file.append("mahalo-share-queue.sqlite");

		// Get a window with MHO_SHARE_TOOLBAR in it.
		var enumerator = this._windowManager.getEnumerator(null);

		while(enumerator.hasMoreElements()) {
			var win = enumerator.getNext();

			if (win.MHO_SHARE_TOOLBAR) {
				MHO_SHARE_TOOLBAR = win.MHO_SHARE_TOOLBAR;
				break;
			}
		}
		
		if (MHO_SHARE_TOOLBAR) {
			var storageService = Components.classes["@mozilla.org/storage/service;1"]
			                        .getService(Components.interfaces.mozIStorageService);
			
			var mDBConn = storageService.openDatabase(file);
		
			mDBConn.executeSimpleSQL("CREATE TABLE IF NOT EXISTS queue (id INTEGER PRIMARY KEY, service TEXT, uri TEXT, title TEXT, description TEXT, tags TEXT, serp TEXT, tryagain INTEGER)");

			// Only select one URI per service so that we don't flood it and get pinged for abuse
			var select = mDBConn.createStatement("SELECT id, service, uri, title, description, tags, serp FROM queue WHERE tryagain < ?1 GROUP BY service");
			select.bindInt64Parameter(0, (new Date().getTime()));
		
			var idsToDelete = [];
		
			try {
				while (select.executeStep()) {
					idsToDelete.push(select.getInt64(0));
				
					MHO_SHARE_TOOLBAR.delaySend(select.getString(1), select.getString(2), select.getString(3), select.getString(4), select.getString(5), select.getString(6));
				}
			} catch (e) {
				logMhoShareMsg2(e);
			} finally {
				select.reset();
			}
		
			if (idsToDelete.length > 0) {
				mDBConn.executeSimpleSQL("DELETE FROM queue WHERE id IN (" + idsToDelete.join(",") + ")");
			}
			
			var howMany = mDBConn.createStatement("SELECT COUNT(*) FROM queue");
			
			try {
				while (howMany.executeStep()) {
					var num = howMany.getInt32(0);
					
					if (num == 0) {
						this.hideBookmarkingAlert();
						break;
					}
					else {
						this.showBookmarkingAlert();
						break;
					}
				}
			} finally {
				howMany.reset();
			}
			
			try { mDBConn.close(); } catch (e) { }
		}
		
		this._checkTimer.initWithCallback(this, this.checkTime, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
    },
	
	hideBookmarkingAlert : function () {
		var enumerator = this._windowManager.getEnumerator(null);

		while(enumerator.hasMoreElements()) {
			var win = enumerator.getNext();
			
			try {
				if (win.document.getElementById("mahalo-share-share-error")) {
					win.document.getElementById("mahalo-share-share-error").setAttribute("disabled", "true");
				}
				if (win.document.getElementById("mahalo-share-share-error-small")) {
					win.document.getElementById("mahalo-share-share-error-small").setAttribute("disabled", "true");
				}
			} catch (e) {
			}
		}
	},
	
	showBookmarkingAlert : function () {
		var enumerator = this._windowManager.getEnumerator(null);

		while(enumerator.hasMoreElements()) {
			var win = enumerator.getNext();
			
			try {
				if (win.document.getElementById("mahalo-share-share-error")) {
					win.document.getElementById("mahalo-share-share-error").setAttribute("disabled", "false");
				}

				if (win.document.getElementById("mahalo-share-share-error-small")) {
					win.document.getElementById("mahalo-share-share-error-small").setAttribute("disabled", "false");
				}
			} catch (e) {
			}
		}
	},
	
	// for nsISupports
	QueryInterface: function(aIID) {
		if (!aIID.equals(Components.interfaces.nsISupports) &&
			!aIID.equals(Components.interfaces.nsIObserver))
			throw Components.results.NS_ERROR_NO_INTERFACE;
		return this;
	},

	// for nsIObserver
	observe: function(subject, topic, data) {
		switch (topic) {
			case "xpcom-shutdown":
				this.destroy();
			break;
		}
	},

	destroy: function(e) {
		if (this._timer) {
			this._timer.cancel();
			this._timer = null;
		}
	}
};

var gMahaloShareTimer = null;

var MahaloShareFactory = {
	createInstance: function (aOuter, aIID)
	{
		if (aOuter != null)
			throw Components.results.NS_ERROR_NO_AGGREGATION;
			
		if (gMahaloShareTimer === null) {
			gMahaloShareTimer = new MahaloShareTimer().QueryInterface(aIID);
		}
		
		return gMahaloShareTimer;
	}
};

var MahaloShareModule = {
	registerSelf: function(aCompMgr, aFileSpec, aLocation, aType) {
		aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
		aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, aFileSpec, aLocation, aType);

		Components.classes["@mozilla.org/categorymanager;1"]
			.getService(Components.interfaces.nsICategoryManager)
			.addCategoryEntry("app-startup",
			CLASS_NAME,
			"service," + CONTRACT_ID,
			true, true);
	},

	unregisterSelf: function(aCompMgr, aLocation, aType)
	{
		aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
		aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);       

		Components.classes["@mozilla.org/categorymanager;1"]
			.getService(Components.interfaces.nsICategoryManager)
			.deleteCategoryEntry("app-startup",
			CLASS_NAME,
			true);
	},

	getClassObject: function(aCompMgr, aCID, aIID)  {
		if (!aIID.equals(Components.interfaces.nsIFactory))
			throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

		if (aCID.equals(CLASS_ID))
			return MahaloShareFactory;

		throw Components.results.NS_ERROR_NO_INTERFACE;
	},

	canUnload: function(aCompMgr) { return true; }
};

function NSGetModule(aCompMgr, aFileSpec) { return MahaloShareModule; }

function logMhoShareMsg2(message) {
	var consoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
	consoleService.logStringMessage("Mahalo Share: " + message);
}