var MHO_SHARE_API = {
	get apiUrl() { return MHO_SHARE_API.prefs.getCharPref("uris.api") + "?mode="; },
	
	get refID() { var rv = '-1'; try { rv = MHO_SHARE_API.prefs.getCharPref("refID"); } catch (e) { } if (rv == '') { rv = '-1'; } return rv; },
	get extID() { try { return MHO_SHARE_API.prefs.getCharPref("extID"); } catch (e) { return ''; } },

	get prefs() { return Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.mahalo.share.") ; },
	get bmPrefs() { return Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.mahalo.bookmarking.") ; },
	
	timeout : 45,
	
	create_request : function (url, sync) {
		var req = new XMLHttpRequest();
		req.parent = this;
		req.open("POST", url, !sync);
		req.setRequestHeader("Content-Type","application/x-www-form-urlencoded");
		req.overrideMimeType("application/xml");
		
		return req;
	},
	
	send_install : function () {
		var url = this.apiUrl + "install";
		
		var req = this.create_request(url);
	
		var xml = '<mahaloapi><refid>' + this.refID + '</refid><extid>' + this.extID + '</extid><version>' + MHOS_VERSION + '</version></mahaloapi>';
	
		req.onreadystatechange = function (event) {
			if (req.readyState == 4){
				try {
					if (MHOS_DEBUG) logMhoShareMsg("Received install response: " + req.responseText);
				
					var xmldoc = req.responseXML;
				
					if (xmldoc.getElementsByTagName("response")[0].firstChild.nodeValue != '1'){
						// Something screwed up.  Try to log this install again later.
						this.prefs.setCharPref("version","");
						
						if (MHOS_DEBUG) logMhoShareMsg("Install ping didn't reach destination.");
						logMhoShareError("Received error from server (install ping): " + req.responseText);
					}
					else {
						logMhoShareError("Received confirmation from server (install ping): " + req.responseText);
					}
				} catch (e) {
					// Didn't ping
					// Try again on next reload
					// Something screwed up.  Try to log this install again later
					this.prefs.setCharPref("version","");
					if (MHOS_DEBUG) logMhoShareMsg("Install ping didn't reach destination.");
					logMhoShareError("Received error from server (install ping): " + req.responseText);
				}
			}
		}
		
		if (MHOS_DEBUG) logMhoShareMsg("Sending install ping: " + xml);
		
		logMhoShareError("Sending install ping: " + xml);
		
		req.send("request="+encodeURIComponent(xml));
	},
	
	send_rating : function (url, score) {
		var apiUrl = this.apiUrl + "rate";
		
		var req = this.create_request(apiUrl);
		
		var xml = '<mahaloapi><refid>' + this.refID + '</refid><extid>' + this.extID + '</extid><url><![CDATA['+url+']]></url><rating>'+score+'</rating></mahaloapi>';
		
		req.onreadystatechange = function (event) {
			if (req.readyState == 4){
				if (MHOS_DEBUG) logMhoShareMsg("Received rating response: " + req.responseText);
			}
		}

		if (MHOS_DEBUG) logMhoShareMsg("Sending rating: " + xml);
		
		req.send("request="+encodeURIComponent(xml));
	},
	
	send_search : function () {
		var url = this.apiUrl + "search";
		
		var req = this.create_request(url);
		
		var xml = '<mahaloapi><refid>' + this.refID + '</refid><extid>' + this.extID + '</extid></mahaloapi>';
		
		req.onreadystatechange = function (event) {
			if (req.readyState == 4){
				if (MHOS_DEBUG) logMhoShareMsg("Received search ping response: " + req.responseText);
				logMhoShareError("Received search ping response: " + req.responseText);
			}
		}
		
		if (MHOS_DEBUG) logMhoShareMsg("Sending search ping: " + xml);
		
		logMhoShareError("Sending search ping: " + xml);
		
		req.send("request="+encodeURIComponent(xml));
	},
	
	send_comparison : function (engine) {
		var url = this.apiUrl + "comparison";
		
		var req = this.create_request(url);
		
		var xml = '<mahaloapi><refid>' + this.refID + '</refid><extid>' + this.extID + '</extid><service><![CDATA['+engine+']]></service></mahaloapi>';
		
		req.onreadystatechange = function (event) {
			if (req.readyState == 4){
				if (MHOS_DEBUG) logMhoShareMsg("Received comparison ping response: " + req.responseText);
				logMhoShareError("Received comparison ping response: " + req.responseText);
			}
		}
		
		if (MHOS_DEBUG) logMhoShareMsg("Sending comparison ping: " + xml);
		
		logMhoShareError("Sending comparison ping: " + xml);
		
		req.send("request="+encodeURIComponent(xml));
	},
	
	get_rating : function (url) {
		var apiUrl = this.apiUrl + "getrating";
		
		var req = this.create_request(apiUrl);
		
		var xml = '<mahaloapi><refid>' + this.refID + '</refid><extid>' + this.extID + '</extid><url><![CDATA['+url+']]></url></mahaloapi>';
		
		req.onreadystatechange = function (event) {
			if (req.readyState == 4){
				if (MHOS_DEBUG) logMhoShareMsg("Received rating: " + req.responseText);
			}
		}
		
		if (MHOS_DEBUG) logMhoShareMsg("Requesting rating: " + xml);
		
		req.send("request="+encodeURIComponent(xml));
	},
	
	setRecovery : function (service, aUri, aTitle, aDescription, aTags, aSerp) {
		MHO_SHARE_API[service].timeout = setTimeout(MHO_SHARE_API.requestFailed, MHO_SHARE_API.timeout * 1000, service, aUri, aTitle, aDescription, aTags, aSerp);
	},
		
	requestFailed : function (service, aUri, aTitle, aDescription, aTags, aSerp) {
		MHO_SHARE_API[service].currentReq.abort();

		// Try again in 3 minutes.
		MHO_SHARE_API.queue(service, aUri, aTitle, aDescription, aTags, aSerp, (new Date().getTime() + (1000 * 60 * 3)));
	},
		
	delicious : {
		uris : {
			add : "https://api.del.icio.us/v1/posts/add?"
		},
		
		timeout : null,
		currentReq : null,
		
		get auth() { return MHO_SHARE_BOOKMARKING.getLogin('http://delicious.mahalo.com/'); },
		
		checkLogin : function (username, password, successCallback, failureCallback) {
			var url = "https://api.del.icio.us/v1/posts/update";
			
			try {
				var req = MHO_SHARE_API.create_request(url);
				req.setRequestHeader("Authorization","Basic " + btoa(username + ":" + password));
				
				req.onreadystatechange = function () {
					try {
						if (req.readyState == 4) {
							if (req.status == 200) {
								successCallback('delicious');
							}
							else {
								failureCallback('delicious');
							}
						}
					} catch (requestAborted) {
						failureCallback('delicious');
					}
				};
				
				req.send(null);				
			} catch (e) {
				failureCallback('delicious');
			}
		},
		
		post : function (aUri, aTitle, aDescription, aTags, aSerp) {
			var auth = this.auth;
			var tags = (aTags) ? aTags.replace(/\s/g, "").split(",").join(" ") : "";
			
			var apiurl = this.uris.add;
			
			var args = { };
			args.description = aTitle;
			args.tags = tags;
			args.url = aUri;
			args.extended = (aDescription) ? aDescription : "";
			
			var postData = MHO_SHARE_API.createPostRequest(args);
			
			var req = MHO_SHARE_API.create_request(apiurl);
			req.setRequestHeader("Content-Type","multipart/form-data; boundary=" + postData.boundary);
			req.setRequestHeader("Content-Length", (postData.requestBody.available()));
			req.setRequestHeader("Authorization","Basic " + btoa(auth.username + ":" + auth.password));
			
			req.onreadystatechange = function (event) {
				if (req.readyState == 4) {
					try {
						if (req.status == 200) {
							var response = req.responseXML.getElementsByTagName("result")[0].getAttribute("code");
				
							if (response == 'done') {
								// No problem.
								clearTimeout(MHO_SHARE_API.delicious.timeout);
							}
						}
					} catch (requestAborted) {
					}
				}
			};
		
			req.send(postData.requestBody);
			this.currentReq = req;
			
			MHO_SHARE_API.setRecovery('delicious', aUri, aTitle, aDescription, aTags, aSerp);
		}
	},
	
	magnolia : {
		uris : {
			add : "https://ma.gnolia.com/api/mirrord/v1/posts/add"
		},
		
		timeout : null,
		currentReq : null,
		
		get auth() { return MHO_SHARE_BOOKMARKING.getLogin('http://magnolia.mahalo.com/'); },
		
		checkLogin : function (username, password, successCallback, failureCallback) {
			var url = "http://ma.gnolia.com/api/mirrord/v1/posts/update";
			
			try {
				var req = MHO_SHARE_API.create_request(url);
				req.setRequestHeader("Authorization","Basic " + btoa(username + ":" + password));
				
				req.onreadystatechange = function () {
					try {
						if (req.readyState == 4) {
							if (req.status == 200) {
								successCallback('magnolia');
							}
							else {
								failureCallback('magnolia');
							}
						}
					} catch (requestAborted) {
						failureCallback('magnolia');
					}
				};
				
				req.send(null);				
			} catch (e) {
				failureCallback('magnolia');
			}
		},
		
		post : function (aUri, aTitle, aDescription, aTags, aSerp) {
			var auth = this.auth;
			
			var apiurl = this.uris.add;
			
			var args = { };
			args.description = aTitle;
			args.tags = aTags;
			args.url = aUri;
			args.extended = (aDescription) ? aDescription : "";
			
			var req = MHO_SHARE_API.create_request(apiurl);
			
			var postData = MHO_SHARE_API.createPostRequest(args);
			req.setRequestHeader("Content-Type","multipart/form-data; boundary=" + postData.boundary);
			req.setRequestHeader("Content-Length", (postData.requestBody.available()));
			req.setRequestHeader("Authorization","Basic " + btoa(auth.username + ":" + auth.password));
			
			req.onreadystatechange = function (event) {
				if (req.readyState == 4) {
					try {
						if (req.status == 200) {
							var response = req.responseXML.getElementsByTagName("result")[0].getAttribute("code");
				
							if (response == 'done') {
								// No problem.
								clearTimeout(MHO_SHARE_API.magnolia.timeout);
							}
						}
					} catch (requestAborted) {
					}
				}
			};
			
			req.send(postData.requestBody);
			this.currentReq = req;
			
			MHO_SHARE_API.setRecovery('magnolia', aUri, aTitle, aDescription, aTags, aSerp);
		}
	},
	
	twitter : {
		uris : {
			add : "http://twitter.com/statuses/update.xml"
		},
		
		template : "%s: %s",
		timeout : null,
		
		timeout : null,
		currentReq : null,
		
		get auth() { return MHO_SHARE_BOOKMARKING.getLogin('http://twitter.mahalo.com/'); },
		
		checkLogin : function (username, password, successCallback, failureCallback) {
			var url = "http://twitter.com/account/verify_credentials.xml";
			
			try {
				var req = MHO_SHARE_API.create_request(url);
				req.setRequestHeader("Authorization","Basic " + btoa(username + ":" + password));
				
				req.onreadystatechange = function () {
					try {
						if (req.readyState == 4) {
							if (req.status == 200) {
								successCallback('twitter');
							}
							else {
								failureCallback('twitter');
							}
						}
					} catch (e) {
						failureCallback('twitter');
					}
				};
				
				req.send(null);				
			} catch (e) {
				failureCallback('twitter');
			}
		},		
		
		post : function (uri, title, description, tags, serp) {
			// Get a tinyURL
			var req = new XMLHttpRequest();
			req.parent = this;
			req.open("GET", "http://tinyurl.com/api-create.php?url=" + uri, true);
			
			try {
				req.onreadystatechange = function (event) {
					if (req.readyState == 4) {
						try {
							if (req.status == 200) {
								uri = req.responseText;
							}

							// A tinyURL outage doesn't mean we won't post, it just means that Twitter will have to convert the URI
							MHO_SHARE_API.twitter.postWithTinyURL(uri, title, description);
						} catch (requestAborted) {
							
						}
					}
				};
			} catch (e) {
				MHO_SHARE_API.twitter.postWithTinyURL(uri, title, description);
			}
			
			req.send(null);
			this.currentReq = req;
			
			MHO_SHARE_API.setRecovery('twitter', uri, title, description, tags, serp);
		},
		
		postWithTinyURL : function (aUri, aTitle, aDescription, aTags, aSerp) {
			var auth = this.auth;
			
			var args = {};
			
			var status = this.template;
			
			if (aDescription) {
				status = status.replace("%s", aDescription.substring(0, 140 - 27));
			}
			else {
				status = status.replace("%s", aTitle.substring(0, 140 - 27));
			}

			var apiurl = this.uris.add;
			
			status = status.replace("%s", aUri);

			args.status = status;
			args.source = 'mahalo';
			
			var req = MHO_SHARE_API.create_request(apiurl);

			var postData = MHO_SHARE_API.createPostRequest(args);
			req.setRequestHeader("Content-Type","multipart/form-data; boundary=" + postData.boundary);
			req.setRequestHeader("Content-Length", (postData.requestBody.available()));
			req.setRequestHeader("Authorization","Basic " + btoa(auth.username + ":" + auth.password));
			
			req.onreadystatechange = function (event) {
				if (req.readyState == 4) {
					try {
						if (req.status == 200) {
							if (req.responseXML.getElementsByTagName("created_at")[0]) {
								clearTimeout(MHO_SHARE_API.twitter.timeout);
							}
						}
					} catch (requestAborted) {
					}
				}
			};

			req.send(postData.requestBody);
			this.currentReq = req;
		}
	},
	
	tumblr : {
		uris : {
			add : "http://www.tumblr.com/api/write"
		},

		timeout : null,
		currentReq : null,
		
		get auth() { return MHO_SHARE_BOOKMARKING.getLogin('http://tumblr.mahalo.com/'); },

		checkLogin : function (username, password, successCallback, failureCallback) {
			var url = "http://www.tumblr.com/api/write";
			
			var args = {};
			args.action = 'authenticate';
			args.email = username;
			args.password = password;
			
			var postData = MHO_SHARE_API.createPostRequest(args);
			
			try {
				var req = MHO_SHARE_API.create_request(url);
				req.overrideMimeType("text/plain");
				req.setRequestHeader("Content-Type","multipart/form-data; boundary=" + postData.boundary);
				req.setRequestHeader("Content-Length", (postData.requestBody.available()));

				req.onreadystatechange = function () {
					try {
						if (req.readyState == 4) {
							if (req.status == 200) {
								successCallback('tumblr');
							}
							else {
								failureCallback('tumblr');
							}
						}
					} catch (e) {
						failureCallback('tumblr');
					}
				};

				req.send(postData.requestBody);		
			} catch (e) {
				failureCallback('tumblr');
			}
		},		

		post : function (aUri, aTitle, aDescription, aTags, aSerp) {
			var auth = this.auth;
			
			var args = {};
			args.email = auth.username;
			args.password = auth.password;
			args.type = 'link';
			args.name = aTitle;
			args.url = aUri;
			args.description = aDescription;

			var apiurl = this.uris.add;
			var req = MHO_SHARE_API.create_request(apiurl);
			req.overrideMimeType("text/plain");
			
			var postData = MHO_SHARE_API.createPostRequest(args);
			req.setRequestHeader("Content-Type","multipart/form-data; boundary=" + postData.boundary);
			req.setRequestHeader("Content-Length", (postData.requestBody.available()));

			req.onreadystatechange = function (event) {
				if (req.readyState == 4) {
					try {
						if (req.status == 201) {
							// No problem.
							clearTimeout(MHO_SHARE_API.tumblr.timeout);
						}
					} catch (requestAborted) {
					}
				}
			};

			MHO_SHARE_API.setRecovery('tumblr', aUri, aTitle, aDescription, aTags, aSerp);
			req.send(postData.requestBody);
			this.currentReq = req;
		}
	},
	
	gbookmarks : {
		uris : {
			login : "https://www.google.com/accounts/ClientLogin",
			add : "http://www.google.com/bookmarks/mark?sig=%s"
		},

		timeout : null,
		currentReq : null,
		
		get auth() { return MHO_SHARE_BOOKMARKING.getLogin('http://gbookmarks.mahalo.com/'); },

		checkLogin : function (username, password, successCallback, failureCallback) {
			var url = this.uris.login;
			
			var args = {};
			args.Email = username;
			args.Passwd = password;
			args.service = 'bookmarks';
			
			var postData = MHO_SHARE_API.createPostRequest(args);

			try {
				var req = MHO_SHARE_API.create_request(url);
				req.setRequestHeader("Content-Type","multipart/form-data; boundary=" + postData.boundary);
				req.setRequestHeader("Content-Length", (postData.requestBody.available()));
				req.overrideMimeType("text/plain");
				
				req.onreadystatechange = function () {
					try {
						if (req.readyState == 4) {
							if (req.status == 200) {
								// Set a cookie.
								
								function addCookie(name, value) {
									var uri = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURI);
									var cservice = Components.classes["@mozilla.org/cookieService;1"].getService().QueryInterface(Components.interfaces.nsICookieService);
									uri.spec = "http://google.com/";
									var cookieString = '';
									cookieString += name + '=' + value + ';';
									cookieString += 'domain=google.com;';
									cookieString += 'path=/;';
									
									cservice.setCookieString(uri, null, cookieString, null);
								}
								
								var cookieString = req.responseText.replace(/^\s+|\s+$/g, "").split("\n");
								for (var i = 0; i < cookieString.length; i++){
									var parts = cookieString[i].split("=");
									addCookie(parts[0], parts[1]);
								}
								
								successCallback('gbookmarks');
							}
							else {
								failureCallback('gbookmarks');
							}
						}
					} catch (e) {
						failureCallback('gbookmarks');
					}
				};

				req.send(postData.requestBody);	
				this.currentReq = req;			
			} catch (e) {
				failureCallback('gbookmarks');
			}
		},		

		post : function (aUri, aTitle, aDescription, aTags, aSerp) {
			// Login, then post.
			
			function successCallback(service) {
				function successCallback2(sig) {
					var args = {};
			
					args.title = aTitle;
					args.bkmk = aUri;
					args.labels = aTags;
					args.annotation = aDescription;
					args.btnA = "Add bookmark";
				
					var apiurl = MHO_SHARE_API.gbookmarks.uris.add.replace("%s", sig);
					var req = MHO_SHARE_API.create_request(apiurl);
					var argPairs = [];

					for (var i in args) {
						argPairs.push(i+"="+encodeURIComponent(args[i]));
					}

					var argString = argPairs.join("&");

					req.setRequestHeader("Content-Type","application/x-www-form-urlencoded");
					req.setRequestHeader("Content-Length",argString.length);
					req.overrideMimeType("text/plain");
					
					req.onreadystatechange = function (event) {
						if (req.readyState == 4) {
							try {
								if (req.status == 200) {
									clearTimeout(MHO_SHARE_API.gbookmarks.timeout);
								}
							} catch (requestAborted) {
							}
						}
					};
					
					req.send(argString);
					MHO_SHARE_API.gbookmarks.currentReq = req;
				}
				
				try {
					var req = new XMLHttpRequest();
					req.open("GET", "http://www.google.com/bookmarks/", true);

					req.onreadystatechange = function () {
						try {
							if (req.readyState == 4) {
								if (req.status == 200) {
									var sig = req.responseText.match(/\/bookmarks\/mark\?op=modlabel&sig=([^"]+)"/im)[1];
									successCallback2(sig);
								}
							}
						} catch (e) {
							logMhoShareMsg(e);
						}
					};

					req.send(null);	
					MHO_SHARE_API.gbookmarks.currentReq = req;			
				} catch (e) {
					logMhoShareMsg(e);
				}
			}
			
			var auth = this.auth;
			MHO_SHARE_API.setRecovery('gbookmarks', aUri, aTitle, aDescription, aTags, aSerp);
			this.checkLogin(auth.username, auth.password, successCallback, function () { });
		}
	},
	
	jaiku : {
		uris : {
			add : "http://api.jaiku.com/json",
			apikey : "http://api.jaiku.com/",
			login : "http://jaiku.com/login"
		},
		
		apikey : null,
		
		timeout : null,
		currentReq : null,
		
		get auth() { return MHO_SHARE_BOOKMARKING.getLogin('http://jaiku.mahalo.com/'); },

		checkLogin : function (username, password, successCallback, failureCallback) {
			this.apikey = null;
			
			var auth = this.auth;
			
			var args = {};
	
			args.log = username;
			args.pwd = password;
			args.redirect_to = this.uris.apikey;
			
			var argPairs = [];

			for (var i in args) {
				argPairs.push(i+"="+encodeURIComponent(args[i]));
			}

			var argString = argPairs.join("&");

			try {
				var req = new XMLHttpRequest();
				req.open("POST", "http://jaiku.com/login", true);
				req.setRequestHeader("Content-Type","application/x-www-form-urlencoded");
				req.setRequestHeader("Content-Length", argString.length);
				req.setRequestHeader("Referer", "http://jaiku.com/login");
				
				req.onreadystatechange = function () {
					try {
						if (req.readyState == 4){
							if (req.status == 200) {
								try {
									var apikey = req.responseText.match(/<strong><code>([a-z0-9]+)<\/code><\/strong>/i)[1];
						
									if (apikey) {
										this.apikey = apikey;
										successCallback('jaiku');
									}
								} catch (e) {
									failureCallback('jaiku');
								}
							}
							else {
								failureCallback('jaiku');
							}
						}
					} catch (e) {
						failureCallback('jaiku');
					}
				};
			
				req.send(argString);
				MHO_SHARE_API.jaiku.currentReq = req;
			} catch (e) {
				failureCallback('jaiku');
			}
		},		

		post : function (aUri, aTitle, aDescription, aTags, aSerp) {
			function doPost(service) {
				// this.apikey is now set
				var auth = MHO_SHARE_API.jaiku.auth;

				var args = {};

				args.user = auth.username;
				args.personal_key = this.apikey;
				args.method = "presence.send";
				
				if (aDescription) {
					args.message = aDescription + ": " + aUri;
				}
				else {
					args.message = aTitle + ": " + aUri;
				}

				var argPairs = [];

				for (var i in args) {
					argPairs.push(i+"="+encodeURIComponent(args[i]));
				}

				var argString = argPairs.join("&");
				var req = new XMLHttpRequest();
				req.open("POST", MHO_SHARE_API.jaiku.uris.add, true);
				req.setRequestHeader("Content-Type","application/x-www-form-urlencoded");
				req.setRequestHeader("Content-Length",argString.length);
				
				req.onreadystatechange = function (event) {
					if (req.readyState == 4) {
						try {
							if (req.status == 200) {
								var json = eval("(" + req.responseText + ")");
								
								if (json.status == 'ok') {
									clearTimeout(MHO_SHARE_API.jaiku.timeout);
								}
							}
						} catch (requestAborted) {
						}
					}
				};
				
				req.send(argString);
				MHO_SHARE_API.jaiku.currentReq = req;
			}
			
			var auth = this.auth;			
			MHO_SHARE_API.setRecovery('jaiku', aUri, aTitle, aDescription, aTags, aSerp);
			this.checkLogin(auth.username, auth.password, doPost, function () { });
		}
	},	
		
	facebook : {
		timeout : null,
		currentReq : null,
		
		get auth() { return MHO_SHARE_BOOKMARKING.getLogin('http://facebook.mahalo.com/'); },
		
		checkLogin : function (username, password, successCallback, failureCallback) {
			// Hit the logout page first to clear any cookies.
			var logoutReq = new XMLHttpRequest();
			logoutReq.open("POST", "http://www.facebook.com/logout.php", true);
			
			var args = {};
			args.confirm = 1;
			
			var postData = MHO_SHARE_API.createPostRequest(args);
			logoutReq.setRequestHeader("Content-Type","multipart/form-data; boundary=" + postData.boundary);
			logoutReq.setRequestHeader("Content-Length", (postData.requestBody.available()));
			
			logoutReq.onreadystatechange = function () {
				if (logoutReq.readyState == 4) {
					var url = "https://login.facebook.com/login.php";
			
					// Get a login page to get a challenge token
					var tokenReq = new XMLHttpRequest();
					tokenReq.open("GET", "http://www.facebook.com/", true);
			
					tokenReq.onreadystatechange = function () {
						if (tokenReq.readyState == 4) {
							var token = tokenReq.responseText.match(/challenge.*([a-z0-9]{32})/i)[1];
							
							var args = {};
							args.challenge = token;
							args.md5pass = 1;
							args.noerror = 1;
							args.email = username;
							args.pass = password;
							args.doquicklogin = 'Login';
					
							var postData = MHO_SHARE_API.createPostRequest(args);
					
							var loginReq = MHO_SHARE_API.create_request(url);
							loginReq.overrideMimeType("text/html");
							loginReq.setRequestHeader("Content-Type","multipart/form-data; boundary=" + postData.boundary);
							loginReq.setRequestHeader("Content-Length", (postData.requestBody.available()));

							loginReq.onreadystatechange = function () {
								if (loginReq.readyState == 4) {
									if (loginReq.responseText.match(/Facebook \| Home/)) {
										successCallback('facebook');
									}
									else {
										failureCallback('facebook');
									}
								}
							};
					
							loginReq.send(postData.requestBody);
							MHO_SHARE_API.facebook.currentReq = loginReq;
						}
					};
			
					tokenReq.send(null);
					MHO_SHARE_API.facebook.currentReq = tokenReq;
				}
			}
			
			logoutReq.send(postData.requestBody);
			MHO_SHARE_API.facebook.currentReq = logoutReq;
		},
		
		post : function (aUri, aTitle, aDescription, aTags, aSerp) {
			// Login.
			
			function post2(service) {
				// Request the form to get the form id.
				var url = "http://www.facebook.com/sharer.php?src=bm&u=" + encodeURIComponent(aUri) + "&t=" + encodeURIComponent(aTitle);
		
				var tokenReq = new XMLHttpRequest();
				tokenReq.open("GET", url, true);
		
				tokenReq.onreadystatechange = function () {
					if (tokenReq.readyState == 4) {
						try {
							if (tokenReq.status == 200) {
								var token = tokenReq.responseText.match(/post_form_id.*([a-z0-9]{32})/i)[1];
								var appid = tokenReq.responseText.match(/appid.*([0-9]{5,})"/i)[1];
						
								var args = {};
								args.action = 'post';
								args.post_form_id = token;
								args["attachment[params][url]"] = aUri;
								args["attachment[params][medium]"] = '102';
								args["attachment[params][title]"] = aTitle;
								args["attachment[params][summary]"] = 'Shared via Mahalo Share for search result http://mahalo.com/' + aSerp.replace(/\s/g, "_");
								args["attachment[params][favicon]"] = '';
								args["attachment[params][extra]"] = '';
								args["attachment[params][video]"] = '';
								args["attachment[params][music]"] = '';
								args["attachment[params][error]"] = '1';
								args["attachment[params][images][0]"] = 'http://mahalo.websnapr.com/?size=T&url=' + encodeURIComponent(aUri);
								args["attachment[type]"] = 100;
								args.is_popup = 1;
								args.appid = appid;
								args.message = aDescription;
							
								var url = "http://www.facebook.com/ajax/share.php";
								
								var argstring = "";
								
								for (var i in args) {
									argstring += encodeURIComponent(i) + "=" + encodeURIComponent(args[i]) + "&";
								}
								
								argstring += "post_form_id=" + args.post_form_id;
								
								var postReq = new XMLHttpRequest();
								postReq.open("POST", url, true);
								postReq.setRequestHeader("Content-Type","application/x-www-form-urlencoded; charset=UTF-8");
								postReq.setRequestHeader("Content-Length", argstring.length);

								postReq.onreadystatechange = function () {
									if (postReq.readyState == 4) {
										try {
											if (postReq.status == 200) {
												clearTimeout(MHO_SHARE_API.facebook.timeout);
											}
										} catch (requestAborted) {
										}
									}
								};
						
								postReq.send(argstring);
								MHO_SHARE_API.facebook.currentReq = postReq;
							}
						} catch (requestAborted) {
						}
					}
				}
				
				tokenReq.send(null);
				MHO_SHARE_API.facebook.currentReq = tokenReq;
			}
			
			var auth = this.auth;
			
			MHO_SHARE_API.setRecovery('facebook', aUri, aTitle, aDescription, aTags, aSerp);
			this.checkLogin(auth.username, auth.password, post2, function () { } );
		}
	},
	
	friendfeed : {
		uris : {
			add : "http://friendfeed.com/api/share"
		},
		
		timeout : null,
		currentReq : null,
		
		get auth() { return MHO_SHARE_BOOKMARKING.getLogin('http://friendfeed.mahalo.com/'); },
		
		checkLogin : function (username, password, successCallback, failureCallback) {
			var url = "http://friendfeed.com/api/validate";
			
			try {
				var req = new XMLHttpRequest();
				req.open("GET", url, true);
				req.setRequestHeader("Content-Length","0");
				req.setRequestHeader("Authorization","Basic " + btoa(username + ":" + password));
				
				req.onreadystatechange = function () {
					try {
						if (req.readyState == 4) {
							if (req.status == 200) {
								successCallback('friendfeed');
							}
							else {
								failureCallback('friendfeed');
							}
						}
					} catch (e) {
						failureCallback('friendfeed');
					}
				};
				
				req.send(null);				
			} catch (e) {
				failureCallback('friendfeed');
			}
		},
		
		post : function (aUri, aTitle, aDescription, aTags, aSerp) {
			var auth = this.auth;
			var apiurl = this.uris.add;
			
			var args = { };
			args.title = aTitle;
			args.link = aUri;
			
			if (aDescription) {
				args.comment = aDescription;
			}
			
			var argstring = "";
			
			for (var i in args) {
				argstring += i + "=" + encodeURIComponent(args[i]) + "&";
			}
			
			var req = new XMLHttpRequest();
			req.open("POST", apiurl, true);
			req.setRequestHeader("Content-Type","application/x-www-form-urlencoded");
			req.setRequestHeader("Content-Length",argstring.length);
			var authString = "Basic " + btoa(auth.username + ":" + auth.password);
			req.setRequestHeader("Authorization",authString);
			
			req.onreadystatechange = function (event) {
				if (req.readyState == 4) {
					try {
						if (req.status == 200) {
							clearTimeout(MHO_SHARE_API.friendfeed.timeout);
						}
					} catch (requestAborted) {
					}
				}
			};
			
			MHO_SHARE_API.setRecovery('friendfeed', aUri, aTitle, aDescription, aTags, aSerp);
			
			req.send(argstring);
			MHO_SHARE_API.friendfeed.currentReq = req;
		}
	},
	
	faves : {
		uris : {
			add : "https://secure.faves.com/v1/posts/add?"
		},
		
		timeout : null,
		currentReq : null,
		
		get auth() { return MHO_SHARE_BOOKMARKING.getLogin('http://faves.mahalo.com/'); },
		
		checkLogin : function (username, password, successCallback, failureCallback) {
			var url = "https://secure.faves.com/v1/posts/update";
			
			try {
				var req = MHO_SHARE_API.create_request(url);
				req.setRequestHeader("Content-Length","0");
				req.setRequestHeader("Authorization","Basic " + btoa(username + ":" + password));
				
				req.onreadystatechange = function () {
					try {
						if (req.readyState == 4) {
							if (req.status == 200) {
								successCallback('faves');
							}
							else {
								failureCallback('faves');
							}
						}
					} catch (e) {
						failureCallback('faves');
					}
				};
				
				req.send(null);				
			} catch (e) {
				failureCallback('faves');
			}
		},
		
		post : function (aUri, aTitle, aDescription, aTags, aSerp) {
			var auth = this.auth;
			
			var apiurl = this.uris.add;
			
			var args = { };
			args.tags = aTags;
			args.description = aTitle;
			args.extended = (aDescription) ? aDescription : "";
			args.url = aUri;
			
			for (var i in args) {
				apiurl += i + "=" + encodeURIComponent(args[i]) + "&";
			}
			
			var req = MHO_SHARE_API.create_request(apiurl);
			
			req.setRequestHeader("Content-Length", 0);
			req.setRequestHeader("Authorization","Basic " + btoa(auth.username + ":" + auth.password));
			
			req.onreadystatechange = function (event) {
				if (req.readyState == 4) {
					try {
						if (req.status == 200) {
							clearTimeout(MHO_SHARE_API.faves.timeout);
						}
					} catch (requestAborted) {
					}
				}
			};
			
			MHO_SHARE_API.setRecovery('faves', aUri, aTitle, aDescription, aTags, aSerp);
			
			req.send(null);
			MHO_SHARE_API.faves.currentReq = req;
		}
	},
	
	myspace : {
		timeout : null,
		currentReq : null,
		
		get auth() { return MHO_SHARE_BOOKMARKING.getLogin('http://myspace.mahalo.com/'); },
		
		checkLogin : function (username, password, successCallback, failureCallback, isLoggedOut) {
			// Get a login page to get a challenge token
			var tokenReq = new XMLHttpRequest();
			
			tokenReq.open("GET", "http://www.myspace.com/", true);
			tokenReq.setRequestHeader("Referer", "http://www.myspace.com/");
			
			tokenReq.onreadystatechange = function () {
				if (tokenReq.readyState == 4) {
					try {
						if (tokenReq.responseText.match(/>Sign Out</)) {
							throw "Logged in.";
						}
						
						var token = tokenReq.responseText.match(/VIEWSTATE" value="([^"]+)"/)[1];
					} catch (e) {
						if (!tokenReq.responseText.match(/>SignUp</)) {
							var logoutReq = new XMLHttpRequest();
							logoutReq.open("GET", "http://collect.myspace.com/index.cfm?fuseaction=signout", true);
							logoutReq.onreadystatechange = function () {
								if (logoutReq.readyState == 4) {
									MHO_SHARE_API.myspace.checkLogin(username, password, successCallback, failureCallback, true);
								}
							};
							
							logoutReq.send(null);
							MHO_SHARE_API.myspace.currentReq = logoutReq;
						}
						else {
							// Paradox.
						}
						
						return;
					}
					
					var url = "http://secure.myspace.com/index.cfm?fuseaction=login.process";
					
					var args = {};
					args["__VIEWSTATE"] = token;
					args.NextPage = "";
					args["ctl00$Main$SplashDisplay$ctl00$Email_Textbox"] = username;
					args["ctl00$Main$SplashDisplay$ctl00$Password_Textbox"] = password;
					args["ctl00$Main$SplashDisplay$ctl00$Login_ImageButton.x"] = 12;
					args["ctl00$Main$SplashDisplay$ctl00$Login_ImageButton.y"] = 2;
					args["ctl00$Main$SplashDisplay$ctl00$nexturl"] = "";
					args["ctl00$Main$SplashDisplay$ctl00$apikey"] = "";
					args["ctl00$Main$SplashDisplay$ctl00$ContainerPage"] = "";
					
					var argstring = "";
					
					for (var i in args) {
						argstring += encodeURIComponent(i) + "=" + encodeURIComponent(args[i]) + "&";
					}
					
					argstring = argstring.substr(0, argstring.length - 1);
					
					var loginReq = MHO_SHARE_API.create_request(url);
					loginReq.overrideMimeType("text/html");
					loginReq.setRequestHeader("Content-Type","application/x-www-form-urlencoded");
					loginReq.setRequestHeader("Content-Length", argstring.length);
					loginReq.setRequestHeader("Referer", "http://www.myspace.com/", true);
					
					loginReq.onreadystatechange = function () {
						if (loginReq.readyState == 4) {
							if (loginReq.responseText.match(/>Sign Out</)) {
								successCallback('myspace');
							}
							else {
								failureCallback('myspace');
							}
						}
					};
			
					loginReq.send(argstring);
					MHO_SHARE_API.myspace.currentReq = loginReq;
				}
			};
	
			tokenReq.send(null);
			MHO_SHARE_API.myspace.currentReq = tokenReq;
		},
		
		post : function (aUri, aTitle, aDescription, aTags, aSerp) {
			var auth = this.auth;
			
			function continuePosting() {
				var url = "http://blog.myspace.com/index.cfm?fuseaction=blog.create&editor=false";
				
				var req = new XMLHttpRequest();
				req.open("GET", url, true);
				req.setRequestHeader("Referer", url);
				
				req.onreadystatechange = function () {
					if (req.readyState == 4) {
						try {
							var nextUrl = req.responseText.match(/(http:\/\/blog\.myspace\.com\/index\.cfm\?fuseaction=blog\.previewBlog&Mytoken=[^']+)'/i)[1];
						} catch (e) {
							return;
						}
						
						var req2 = new XMLHttpRequest();
						req2.open("POST", nextUrl, true);
						req2.setRequestHeader("Referer", url, true);
						
						var today = new Date();
						
						var args = {};
						args.blogID = "-1";
						args.postMonth = today.getMonth() + 1;
						args.postDay = today.getDate();
						args.postYear = today.getFullYear();
						args.postHour = today.getHours();
						args.postMinute = today.getMinutes();
						args.postTimeMarker = (args.postHour > 11) ? "PM" : "AM";
						
						if (args.postHour > 12) args.postHour = args.postHour - 12;
						if (args.postHour == 0) args.postHour = "12";
						
						args.subject = aTitle;
						args.BlogCategoryID = "0"
						args.editor = "false";
						
						args.body = '<p>';
						
						if (aDescription) { 
							args.body += aDescription + '<br /><br />';
						}
						
						args.body += '<a href="' + aUri + '">' + aUri + '</a></p>'
						args.CurrentlyASIN = "";
						args.CurrentlyProductName = "";
						args.CurrentlyProductBy = "";
						args.CurrentlyImageURL = "";
						args.CurrentlyProductURL = "";
						args.CurrentlyProductReleaseDate = "";
						args.CurrentlyProductType = "";
						args.MoodSetID = "7";
						args.MoodID = "0";
						args.MoodOther = "";
						args.BlogViewingPrivacyID = "0";
						args.Enclosure = "";
						
						var argstring = "";
						
						for (var i in args) {
							argstring += encodeURIComponent(i) + "=" + encodeURIComponent(args[i]).replace(/%20/g, "+") + "&";
						}
						
						argstring = argstring.substring(0, argstring.length - 1);
						
						req2.setRequestHeader("Content-Type","application/x-www-form-urlencoded");
						req2.setRequestHeader("Content-Length", argstring.length);
						
						req2.onreadystatechange = function () {
							if (req2.readyState == 4) {
								for (var i in args) {
									var newKey = i.charAt(0).toUpperCase() + i.substring(1);
									var value = args[i];
									
									delete args[i];
									
									args[newKey] = value;
								}
								
								args.isPreviewed = "true";
								
								try {
									var hash = req2.responseText.match(/hash" value="([^"]+)"/i)[1];
								} catch (e) {
									return;
								}
								
								args.hash = hash;
								
								var argstring = "";

								for (var i in args) {
									argstring += encodeURIComponent(i) + "=" + encodeURIComponent(args[i]) + "&";
								}
								
								argstring = argstring.substring(0, argstring.length - 1);
								
								var req3 = new XMLHttpRequest();
								req3.open("POST", "http://blog.myspace.com/index.cfm?fuseaction=blog.processCreate", true);
								req3.setRequestHeader("Referer", nextUrl, true);
								req3.setRequestHeader("Content-Type","application/x-www-form-urlencoded");
								req3.setRequestHeader("Content-Length", argstring.length);
								
								req3.onreadystatechange = function () {
									if (req3.readyState == 4) {
										clearTimeout(MHO_SHARE_API.myspace.timeout);
									}
								};
								
								req3.send(argstring);
							}
						};
						
						req2.send(argstring);
					}
				};
				
				req.send(null);
			}
			
			MHO_SHARE_API.setRecovery('myspace', aUri, aTitle, aDescription, aTags, aSerp);
			
			this.checkLogin(auth.username, auth.password, continuePosting, function () { });
		}
	},
	
	createPostRequest : function (args) {
		const BOUNDARY="---------------------------32191240128944"; 
		
		const MULTI = "@mozilla.org/io/multiplex-input-stream;1";
		const STRINGIS = "@mozilla.org/io/string-input-stream;1";
		const BUFFERED = "@mozilla.org/network/buffered-input-stream;1";
														   
		const nsIMultiplexInputStream = Components.interfaces.nsIMultiplexInputStream;
		const nsIStringInputStream = Components.interfaces.nsIStringInputStream;
		const nsIBufferedInputStream = Components.interfaces.nsIBufferedInputStream;
		
		var sheader = '';
		
		for (var i in args) {
			sheader += "--" + BOUNDARY + "\r\n";
			sheader += "Content-Disposition: form-data; name=\"" + encodeURIComponent(i) + "\"";
			
			if (typeof args[i] == "object") {
				if (args[i].type == 'file') {
					sheader += " filename=\"" + args[i].filename + "\"";
				
					if (args[i].headers.length > 0) {
						for (var q = 0; q < args[i].headers.length; q++){
							sheader += "\r\n" + args[i].headers[q];
						}
					}
				
					sheader += args[i].content;
				}
			}
			
			sheader += "\r\n";
			
			sheader += "\r\n";
			sheader += args[i];
			sheader += "\r\n";
		}
		
		sheader += "--" + BOUNDARY + "--\r\n";
		sheader = encodeToUtf8(sheader);
		var hsis = Components.classes[STRINGIS].createInstance(nsIStringInputStream);
		hsis.setData(sheader,sheader.length);

		var uploadStream = Components.classes[MULTI].createInstance(nsIMultiplexInputStream);
		uploadStream.appendStream(hsis);
		
		return { "requestBody" : uploadStream, "boundary": BOUNDARY };
		
		function encodeToUtf8(oStr) {
			var utfStr = oStr;
			var uConv = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter); 
			uConv.charset = "UTF-8";	 	 
			utfStr = uConv.ConvertFromUnicode (oStr);	 	 
			return utfStr;
		}
	},
	
	queue : function (service, uri, title, description, tags, serp, tryagain) {
		try {
			var mDBConn = this.getDB();
			
			mDBConn.executeSimpleSQL("CREATE TABLE IF NOT EXISTS queue (id INTEGER PRIMARY KEY, service TEXT, uri TEXT, title TEXT, description TEXT, tags TEXT, serp TEXT, tryagain INTEGER)");
		
			var insert = mDBConn.createStatement("INSERT INTO queue (service, uri, title, description, tags, serp, tryagain) VALUES ( ?1, ?2, ?3, ?4, ?5, ?6, ?7)");
			insert.bindUTF8StringParameter(0, service);
			insert.bindUTF8StringParameter(1, uri);
			insert.bindUTF8StringParameter(2, title);
			insert.bindUTF8StringParameter(3, description);
			insert.bindUTF8StringParameter(4, tags);
			insert.bindUTF8StringParameter(5, serp);
			insert.bindInt64Parameter(6, tryagain);
			insert.execute();
		
			try { mDBConn.close(); } catch (e) { }
		
			if (document.getElementById("mahalo-share-share-error")) {
				document.getElementById("mahalo-share-share-error").setAttribute("disabled", "false");
			}
		
			if (document.getElementById("mahalo-share-share-error-small")) {
				document.getElementById("mahalo-share-share-error-small").setAttribute("disabled", "false");
			}
		} catch (e) {
			// SQLite file is probably borked.
			// Do nothing for now, since that may not actually be the case.
		}
	},
	
	showQueuedRequests : function () {
		openDialog("chrome://mahalo-share/content/error-dialog.xul", "mahalo-share-errors", "chrome,centerscreen=yes");
	},
	
	populateErrorDialog : function () {
		var mDBConn = this.getDB();
		
		var select = mDBConn.createStatement("SELECT service, uri, title, id FROM queue");
		var queued = [];
		
		try {
			while (select.executeStep()) {
				queued.push({ 'service': select.getString(0), 'title': select.getString(2), 'id': select.getInt32(3) });
			}
		} finally {
			select.reset();
		}
		
		try { mDBConn.close(); } catch (e) { }
		
		var parent = document.getElementById("mahalo-share-errors");
		
		while (parent.lastChild.nodeName == 'listitem') parent.removeChild(parent.lastChild);
		
		for (var i = 0; i < queued.length; i++) {
			var item = document.createElement('listitem');
			var o1 = document.createElement('listcell');
			var o2 = document.createElement('listcell');
			o1.setAttribute("label", queued[i].service);
			o2.setAttribute("label", queued[i].title);

			item.setAttribute("value", queued[i].id);

			item.appendChild(o1);
			item.appendChild(o2);
			
			parent.appendChild(item);
		}
	},
	
	cancelSelectedItems : function () {
		var mDBConn = this.getDB();
		var list = document.getElementById('mahalo-share-errors');
		
		var items = list.selectedItems;

		for (var i = items.length - 1; i >= 0; i--){
			if (items[i].nodeName == 'listitem'){
				mDBConn.executeSimpleSQL("DELETE FROM queue WHERE id=" + items[i].getAttribute("value"));
				list.removeItemAt(list.getIndexOfItem(items[i]));
			}
		}
		
		try { mDBConn.close(); } catch (e) { }
		
		var itemsLeft = list.getElementsByTagName("listitem");
		
		if (itemsLeft.length == 0) {
			if (window.opener.document.getElementById("mahalo-share-share-error")) {
				window.opener.document.getElementById("mahalo-share-share-error").setAttribute("disabled", "true");
			}
			
			if (window.opener.document.getElementById("mahalo-share-share-error-small")) {
				window.opener.document.getElementById("mahalo-share-share-error-small").setAttribute("disabled", "true");
			}
		}
	},	
	
	getDB : function () {
		var file = Components.classes["@mozilla.org/file/directory_service;1"]
		                     .getService(Components.interfaces.nsIProperties)
		                     .get("ProfD", Components.interfaces.nsIFile);
		file.append("mahalo-share-queue.sqlite");

		var storageService = Components.classes["@mozilla.org/storage/service;1"]
		                        .getService(Components.interfaces.mozIStorageService);
		var mDBConn = storageService.openDatabase(file);
		return mDBConn;
	}
};
